@@ -7,7 +7,7 @@
 // (Tracked in [issue #7625].)
 //
 //  1. ✅ Minimal support for the [external special remote protocol]. Tested on
-//     "local" and "drive" backends.
+//     "local", "drive", and "dropbox" backends.
 //  2. Add support for the ASYNC protocol extension. This may improve performance.
 //  3. Support the [simple export interface]. This will enable `git-annex
 //     export` functionality.
@@ -107,35 +107,6 @@ func (m *messageParser) finalParameter() string {
 	return param
 }
 
-// configDefinition describes a configuration value required by this command. We
-// use "GETCONFIG" messages to query git-annex for these values at runtime.
-type configDefinition struct {
-	names        []string
-	description  string
-	destination  *string
-	defaultValue *string
-}
-
-func (c *configDefinition) getCanonicalName() string {
-	if len(c.names) < 1 {
-		panic(fmt.Errorf("configDefinition must have at least one name: %v", c))
-	}
-	return c.names[0]
-}
-
-// fullDescription returns a single-line, human-readable description for this
-// config. The returned string begins with a list of synonyms and ends with
-// `c.description`.
-func (c *configDefinition) fullDescription() string {
-	if len(c.names) <= 1 {
-		return c.description
-	}
-	// Exclude the canonical name from the list of synonyms.
-	synonyms := c.names[1:len(c.names)]
-	commaSeparatedSynonyms := strings.Join(synonyms, ", ")
-	return fmt.Sprintf("(synonyms: %s) %s", commaSeparatedSynonyms, c.description)
-}
-
 // server contains this command's current state.
 type server struct {
 	reader *bufio.Reader
@@ -157,7 +128,7 @@ type server struct {
 }
 
 func (s *server) sendMsg(msg string) {
-	msg = msg + "\n"
+	msg += "\n"
 	if _, err := io.WriteString(s.writer, msg); err != nil {
 		panic(err)
 	}
@@ -260,63 +231,42 @@ func (s *server) run() error {
 }
 
 // Idempotently handle an incoming INITREMOTE message. This should perform
-// one-time setup operations, but we may receive the command again, e.g. when
-// this git-annex remote is initialized in a different repository.
+// one-time setup operations for the remote, such as validating or rejecting
+// config values. We may receive the INITREMOTE message again in later sessions,
+// e.g. when the same git-annex remote is initialized in a different repository.
+// However, we are *not* guaranteed to receive the INITREMOTE message once per
+// session, so do not mutate state here and expect it to always be available in
+// other handler functions.
 func (s *server) handleInitRemote() error {
 	if err := s.queryConfigs(); err != nil {
 		return fmt.Errorf("failed to get configs: %w", err)
 	}
 
-	remoteRootFs, err := cache.Get(context.TODO(), fmt.Sprintf("%s:", s.configRcloneRemoteName))
-	if err != nil {
-		s.sendMsg("INITREMOTE-FAILURE failed to open root directory of rclone remote")
-		return fmt.Errorf("failed to open root directory of rclone remote: %w", err)
+	if err := validateRemoteName(s.configRcloneRemoteName); err != nil {
+		s.sendMsg(fmt.Sprintf("INITREMOTE-FAILURE %s", err))
+		return fmt.Errorf("failed to init remote: %w", err)
 	}
 
-	if !remoteRootFs.Features().CanHaveEmptyDirectories {
-		s.sendMsg("INITREMOTE-FAILURE this rclone remote does not support empty directories")
-		return fmt.Errorf("rclone remote does not support empty directories")
-	}
-
-	if err := operations.Mkdir(context.TODO(), remoteRootFs, s.configPrefix); err != nil {
-		s.sendMsg("INITREMOTE-FAILURE failed to mkdir")
-		return fmt.Errorf("failed to mkdir: %w", err)
+	if mode := parseLayoutMode(s.configRcloneLayout); mode == layoutModeUnknown {
+		err := fmt.Errorf("unknown layout mode: %s", s.configRcloneLayout)
+		s.sendMsg(fmt.Sprintf("INITREMOTE-FAILURE %s", err))
+		return fmt.Errorf("failed to init remote: %w", err)
 	}
 
 	s.sendMsg("INITREMOTE-SUCCESS")
 	return nil
 }
 
-// Get a list of configs with pointers to fields of `s`.
-func (s *server) getRequiredConfigs() []configDefinition {
-	defaultRclonePrefix := "git-annex-rclone"
-	defaultRcloneLayout := "nodir"
-
-	return []configDefinition{
-		{
-			[]string{"rcloneremotename", "target"},
-			"Name of the rclone remote to use. " +
-				"Must match a remote known to rclone. " +
-				"(Note that rclone remotes are a distinct concept from git-annex remotes.)",
-			&s.configRcloneRemoteName,
-			nil,
-		},
-		{
-			[]string{"rcloneprefix", "prefix"},
-			"Directory where rclone will write git-annex content. " +
-				fmt.Sprintf("If not specified, defaults to %q. ", defaultRclonePrefix) +
-				"This directory will be created on init if it does not exist.",
-			&s.configPrefix,
-			&defaultRclonePrefix,
-		},
-		{
-			[]string{"rclonelayout", "rclone_layout"},
-			"Defines where, within the rcloneprefix directory, rclone will write git-annex content. " +
-				fmt.Sprintf("Must be one of %v. ", allLayoutModes()) +
-				fmt.Sprintf("If empty, defaults to %q.", defaultRcloneLayout),
-			&s.configRcloneLayout,
-			&defaultRcloneLayout,
-		},
+func (s *server) mustSetConfigValue(id configID, value string) {
+	switch id {
+	case configRemoteName:
+		s.configRcloneRemoteName = value
+	case configPrefix:
+		s.configPrefix = value
+	case configLayout:
+		s.configRcloneLayout = value
+	default:
+		panic(fmt.Errorf("unhandled configId: %v", id))
 	}
 }
 
@@ -328,8 +278,8 @@ func (s *server) queryConfigs() error {
 
 	// Send a "GETCONFIG" message for each required config and parse git-annex's
 	// "VALUE" response.
-	for _, config := range s.getRequiredConfigs() {
-		var valueReceived bool
+queryNextConfig:
+	for _, config := range requiredConfigs {
 		// Try each of the config's names in sequence, starting with the
 		// canonical name.
 		for _, configName := range config.names {
@@ -345,19 +295,15 @@ func (s *server) queryConfigs() error {
 				return fmt.Errorf("failed to parse config value: %s %s", valueKeyword, message.line)
 			}
 
-			value := message.finalParameter()
-			if value != "" {
-				*config.destination = value
-				valueReceived = true
-				break
+			if value := message.finalParameter(); value != "" {
+				s.mustSetConfigValue(config.id, value)
+				continue queryNextConfig
 			}
 		}
-		if !valueReceived {
-			if config.defaultValue == nil {
-				return fmt.Errorf("did not receive a non-empty config value for %q", config.getCanonicalName())
-			}
-			*config.destination = *config.defaultValue
+		if config.defaultValue == "" {
+			return fmt.Errorf("did not receive a non-empty config value for %q", config.getCanonicalName())
 		}
+		s.mustSetConfigValue(config.id, config.defaultValue)
 	}
 
 	s.configsDone = true
@@ -376,7 +322,7 @@ func (s *server) handlePrepare() error {
 // Git-annex is asking us to return the list of settings that we use. Keep this
 // in sync with `handlePrepare()`.
 func (s *server) handleListConfigs() {
-	for _, config := range s.getRequiredConfigs() {
+	for _, config := range requiredConfigs {
 		s.sendMsg(fmt.Sprintf("CONFIG %s %s", config.getCanonicalName(), config.fullDescription()))
 	}
 	s.sendMsg("CONFIGEND")
