@@ -7,9 +7,9 @@ package config
 
 import (
 	"fmt"
-	"log"
 	"os"
 
+	"github.com/rclone/rclone/fs"
 	"github.com/rclone/rclone/lib/terminal"
 )
 
@@ -22,7 +22,7 @@ func ReadPassword() string {
 	line, err := terminal.ReadPassword(stdin)
 	_, _ = fmt.Fprintln(os.Stderr)
 	if err != nil {
-		log.Fatalf("Failed to read password: %v", err)
+		fs.Fatalf(nil, "Failed to read password: %v", err)
 	}
 	return string(line)
 }
